@@ -1,4 +1,4 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -15,12 +15,15 @@
 import os
 import random
 import sys
+import time
 
-from robot import utils
 from robot.errors import DataError, FrameworkError
 from robot.output import LOGGER, loggerhelper
 from robot.result.keywordremover import KeywordRemover
-from robot.result.flattenkeywordmatcher import FlattenKeywordMatcher
+from robot.result.flattenkeywordmatcher import validate_flatten_keyword
+from robot.utils import (abspath, escape, format_time, get_link_path,
+                         html_escape, is_list_like,
+                         split_args_from_name_or_path)
 
 from .gatherfailed import gather_failed_tests
 
@@ -31,24 +34,23 @@ class _BaseSettings(object):
                  'Metadata'         : ('metadata', []),
                  'TestNames'        : ('test', []),
                  'ReRunFailed'      : ('rerunfailed', 'NONE'),
-                 'DeprecatedRunFailed': ('runfailed', 'NONE'),
+                 'DeprecatedRunFailed': ('runfailed', 'NONE'),  # TODO: Remove in RF 3.0
                  'SuiteNames'       : ('suite', []),
                  'SetTag'           : ('settag', []),
                  'Include'          : ('include', []),
                  'Exclude'          : ('exclude', []),
                  'Critical'         : ('critical', None),
                  'NonCritical'      : ('noncritical', None),
-                 'OutputDir'        : ('outputdir', utils.abspath('.')),
+                 'OutputDir'        : ('outputdir', abspath('.')),
                  'Log'              : ('log', 'log.html'),
                  'Report'           : ('report', 'report.html'),
                  'XUnit'            : ('xunit', None),
-                 'DeprecatedXUnit'  : ('xunitfile', None),
                  'SplitLog'         : ('splitlog', False),
                  'TimestampOutputs' : ('timestampoutputs', False),
                  'LogTitle'         : ('logtitle', None),
                  'ReportTitle'      : ('reporttitle', None),
                  'ReportBackground' : ('reportbackground',
-                                       ('#99FF66', '#99FF66', '#FF3333')),
+                                       ('#9e9', '#9e9', '#f66')),
                  'SuiteStatLevel'   : ('suitestatlevel', -1),
                  'TagStatInclude'   : ('tagstatinclude', []),
                  'TagStatExclude'   : ('tagstatexclude', []),
@@ -57,14 +59,17 @@ class _BaseSettings(object):
                  'TagStatLink'      : ('tagstatlink', []),
                  'RemoveKeywords'   : ('removekeywords', []),
                  'FlattenKeywords'  : ('flattenkeywords', []),
-                 'NoStatusRC'       : ('nostatusrc', False),
-                 'MonitorColors'    : ('monitorcolors', 'AUTO'),
+                 'PreRebotModifiers': ('prerebotmodifier', []),
+                 'StatusRC'         : ('statusrc', True),
+                 'ConsoleColors'    : ('consolecolors', 'AUTO'),
+                 'MonitorColors'    : ('monitorcolors', None),  # TODO: Remove in RF 3.0
                  'StdOut'           : ('stdout', None),
                  'StdErr'           : ('stderr', None),
                  'XUnitSkipNonCritical' : ('xunitskipnoncritical', False)}
     _output_opts = ['Output', 'Log', 'Report', 'XUnit', 'DebugFile']
 
     def __init__(self, options=None, **extra_options):
+        self.start_timestamp = format_time(time.time(), '', '-', '')
         self._opts = {}
         self._cli_opts = self._cli_opts.copy()
         self._cli_opts.update(self._extra_cli_opts)
@@ -73,12 +78,10 @@ class _BaseSettings(object):
     def _process_cli_opts(self, opts):
         for name, (cli_name, default) in self._cli_opts.items():
             value = opts[cli_name] if cli_name in opts else default
-            if default == [] and isinstance(value, basestring):
+            if default == [] and not is_list_like(value):
                 value = [value]
             self[name] = self._process_value(name, value)
         self['TestNames'] += self['ReRunFailed'] or self['DeprecatedRunFailed']
-        if self['DeprecatedXUnit']:
-            self['XUnit'] = self['DeprecatedXUnit']
 
     def __setitem__(self, name, value):
         if name not in self._cli_opts:
@@ -86,8 +89,16 @@ class _BaseSettings(object):
         self._opts[name] = value
 
     def _process_value(self, name, value):
-        if name in ['ReRunFailed', 'DeprecatedRunFailed']:
+        if name == 'ReRunFailed':
             return gather_failed_tests(value)
+        if name == 'DeprecatedRunFailed':
+            if value.upper() != 'NONE':
+                LOGGER.warn('Option --runfailed is deprecated and will be '
+                            'removed in the future. Use --rerunfailed instead.')
+            return gather_failed_tests(value)
+        if name == 'DeprecatedMerge' and value is True:
+            LOGGER.warn('Option --rerunmerge is deprecated and will be '
+                        'removed in the future. Use --merge instead.')
         if name == 'LogLevel':
             return self._process_log_level(value)
         if value == self._get_default_value(name):
@@ -104,15 +115,16 @@ class _BaseSettings(object):
             return [self._format_tag_patterns(v) for v in value]
         if name in self._output_opts and (not value or value.upper() == 'NONE'):
             return None
-        if name == 'DeprecatedXUnit':
-            LOGGER.warn('Option --xunitfile is deprecated. Use --xunit instead.')
-            return self._process_value('XUnit', value)
         if name == 'OutputDir':
-            return utils.abspath(value)
-        if name in ['SuiteStatLevel', 'MonitorWidth']:
+            return abspath(value)
+        if name in ['MonitorWidth', 'MonitorColors', 'MonitorMarkers']:
+            option = '--' + name.lower()
+            LOGGER.warn("Option '%s' is deprecated. Use '%s' instead."
+                        % (option, option.replace('monitor', 'console')))
+        if name in ['SuiteStatLevel', 'MonitorWidth', 'ConsoleWidth']:
             return self._convert_to_positive_integer_or_default(name, value)
-        if name in ['Listeners', 'VariableFiles']:
-            return [self._split_args_from_name_or_path(item) for item in value]
+        if name == 'VariableFiles':
+            return [split_args_from_name_or_path(item) for item in value]
         if name == 'ReportBackground':
             return self._process_report_background(value)
         if name == 'TagStatCombine':
@@ -121,10 +133,6 @@ class _BaseSettings(object):
             return [v for v in [self._process_tag_stat_link(v) for v in value] if v]
         if name == 'Randomize':
             return self._process_randomize_value(value)
-        if name == 'RunMode':
-            LOGGER.warn('Option --runmode is deprecated in Robot Framework 2.8 '
-                        'and will be removed in the future.')
-            return [self._process_runmode_value(v) for v in value]
         if name == 'RemoveKeywords':
             self._validate_remove_keywords(value)
         if name == 'FlattenKeywords':
@@ -176,14 +184,6 @@ class _BaseSettings(object):
         raise DataError("Option '%s' does not support value '%s'."
                         % (option_name, given_value))
 
-    def _process_runmode_value(self, original_value):
-        formatted_value = original_value.lower()
-        if formatted_value not in ('exitonfailure', 'skipteardownonexit',
-                                   'dryrun', 'random:test', 'random:suite',
-                                   'random:all'):
-            self._raise_invalid_option_value('--runmode', original_value)
-        return formatted_value
-
     def __getitem__(self, name):
         if name not in self._opts:
             raise KeyError("Non-existing setting '%s'" % name)
@@ -204,14 +204,14 @@ class _BaseSettings(object):
             LOGGER.error('Log file is not created if output.xml is disabled.')
             return None
         name = self._process_output_name(option, name)
-        path = utils.abspath(os.path.join(self['OutputDir'], name))
+        path = abspath(os.path.join(self['OutputDir'], name))
         self._create_output_dir(os.path.dirname(path), option)
         return path
 
     def _process_output_name(self, option, name):
         base, ext = os.path.splitext(name)
         if self['TimestampOutputs']:
-            base = '%s-%s' % (base, utils.get_start_timestamp('', '-', ''))
+            base = '%s-%s' % (base, self.start_timestamp)
         ext = self._get_output_extension(ext, option)
         return base + ext
 
@@ -230,7 +230,7 @@ class _BaseSettings(object):
         try:
             if not os.path.exists(path):
                 os.makedirs(path)
-        except EnvironmentError, err:
+        except EnvironmentError as err:
             raise DataError("Creating %s file directory '%s' failed: %s"
                             % (type_.lower(), path, err.strerror))
 
@@ -262,6 +262,8 @@ class _BaseSettings(object):
                 pattern = pattern.replace(search, replace)
         while '  ' in pattern:
             pattern = pattern.replace('  ', ' ')
+        if pattern.startswith(' NOT'):
+            pattern = pattern[1:]
         return pattern
 
     def _process_tag_stat_link(self, value):
@@ -285,32 +287,18 @@ class _BaseSettings(object):
     def _get_default_value(self, name):
         return self._cli_opts[name][1]
 
-    def _split_args_from_name_or_path(self, name):
-        if ':' not in name or os.path.exists(name):
-            args = []
-        else:
-            args = name.split(':')
-            name = args.pop(0)
-            # Handle absolute Windows paths with arguments
-            if len(name) == 1 and args[0].startswith(('/', '\\')):
-                name = name + ':' + args.pop(0)
-        if os.path.exists(name):
-            name = os.path.abspath(name)
-        return name, args
-
     def _validate_remove_keywords(self, values):
         for value in values:
             try:
                 KeywordRemover(value)
-            except DataError, err:
+            except DataError as err:
                 raise DataError("Invalid value for option '--removekeywords'. %s" % err)
 
     def _validate_flatten_keywords(self, values):
-        for value in values:
-            try:
-                FlattenKeywordMatcher(value)
-            except DataError, err:
-                raise DataError("Invalid value for option '--flattenkeywords'. %s" % err)
+        try:
+            validate_flatten_keyword(values)
+        except DataError as err:
+            raise DataError("Invalid value for option '--flattenkeywords'. %s" % err)
 
     def __contains__(self, setting):
         return setting in self._cli_opts
@@ -319,6 +307,10 @@ class _BaseSettings(object):
         return '\n'.join('%s: %s' % (name, self._opts[name])
                          for name in sorted(self._opts))
 
+    @property
+    def output_directory(self):
+        return self['OutputDir']
+
     @property
     def output(self):
         return self['Output']
@@ -335,13 +327,17 @@ class _BaseSettings(object):
     def xunit(self):
         return self['XUnit']
 
+    @property
+    def log_level(self):
+        return self['LogLevel']
+
     @property
     def split_log(self):
         return self['SplitLog']
 
     @property
     def status_rc(self):
-        return not self['NoStatusRC']
+        return self['StatusRC']
 
     @property
     def xunit_skip_noncritical(self):
@@ -374,6 +370,14 @@ class _BaseSettings(object):
     def flatten_keywords(self):
         return self['FlattenKeywords']
 
+    @property
+    def pre_rebot_modifiers(self):
+        return self['PreRebotModifiers']
+
+    @property
+    def console_colors(self):
+        return self['MonitorColors'] or self['ConsoleColors']
+
 
 class RobotSettings(_BaseSettings):
     _extra_cli_opts = {'Output'             : ('output', 'output.xml'),
@@ -383,14 +387,19 @@ class RobotSettings(_BaseSettings):
                        'ExitOnError'        : ('exitonerror', False),
                        'SkipTeardownOnExit' : ('skipteardownonexit', False),
                        'Randomize'          : ('randomize', 'NONE'),
-                       'RunMode'            : ('runmode', []),
                        'RunEmptySuite'      : ('runemptysuite', False),
                        'WarnOnSkipped'      : ('warnonskippedfiles', False),
                        'Variables'          : ('variable', []),
                        'VariableFiles'      : ('variablefile', []),
+                       'PreRunModifiers'    : ('prerunmodifier', []),
                        'Listeners'          : ('listener', []),
-                       'MonitorWidth'       : ('monitorwidth', 78),
-                       'MonitorMarkers'     : ('monitormarkers', 'AUTO'),
+                       'ConsoleType'        : ('console', 'verbose'),
+                       'ConsoleTypeDotted'  : ('dotted', False),
+                       'ConsoleTypeQuiet'   : ('quiet', False),
+                       'ConsoleWidth'       : ('consolewidth', 78),
+                       'MonitorWidth'       : ('monitorwidth', 0),  # TODO: Remove in RF 3.0
+                       'ConsoleMarkers'     : ('consolemarkers', 'AUTO'),
+                       'MonitorMarkers'     : ('monitormarkers', None),  # TODO: Remove in RF 3.0
                        'DebugFile'          : ('debugfile', None)}
 
     def get_rebot_settings(self):
@@ -411,7 +420,11 @@ class RobotSettings(_BaseSettings):
         return self.output is None
 
     def _escape_as_data(self, value):
-        return utils.escape(value)
+        return escape(value)
+
+    @property
+    def debug_file(self):
+        return self['DebugFile']
 
     @property
     def suite_config(self):
@@ -424,7 +437,7 @@ class RobotSettings(_BaseSettings):
             'exclude_tags': self['Exclude'],
             'include_suites': self['SuiteNames'],
             'include_tests': self['TestNames'],
-            'empty_suite_ok': self['RunEmptySuite'],
+            'empty_suite_ok': self.run_empty_suite,
             'randomize_suites': self.randomize_suites,
             'randomize_tests': self.randomize_tests,
             'randomize_seed': self.randomize_seed,
@@ -436,23 +449,18 @@ class RobotSettings(_BaseSettings):
 
     @property
     def randomize_suites(self):
-        return (self['Randomize'][0] in ('suites', 'all') or
-                any(mode in ('random:suite', 'random:all') for mode in self['RunMode']))
+        return self['Randomize'][0] in ('suites', 'all')
 
     @property
     def randomize_tests(self):
-        return (self['Randomize'][0] in ('tests', 'all') or
-                any(mode in ('random:test', 'random:all') for mode in self['RunMode']))
+        return self['Randomize'][0] in ('tests', 'all')
 
     @property
     def dry_run(self):
-        return (self['DryRun'] or
-                any(mode == 'dryrun' for mode in self['RunMode']))
-
+        return self['DryRun']
     @property
     def exit_on_failure(self):
-        return (self['ExitOnFailure'] or
-                any(mode == 'exitonfailure' for mode in self['RunMode']))
+        return self['ExitOnFailure']
 
     @property
     def exit_on_error(self):
@@ -460,19 +468,51 @@ class RobotSettings(_BaseSettings):
 
     @property
     def skip_teardown_on_exit(self):
-        return (self['SkipTeardownOnExit'] or
-                any(mode == 'skipteardownonexit' for mode in self['RunMode']))
+        return self['SkipTeardownOnExit']
 
     @property
-    def console_logger_config(self):
+    def console_output_config(self):
         return {
-            'width':   self['MonitorWidth'],
-            'colors':  self['MonitorColors'],
-            'markers': self['MonitorMarkers'],
+            'type':    self.console_type,
+            'width':   self.console_width,
+            'colors':  self.console_colors,
+            'markers': self.console_markers,
             'stdout':  self['StdOut'],
             'stderr':  self['StdErr']
         }
 
+    @property
+    def console_type(self):
+        if self['ConsoleTypeQuiet']:
+            return 'quiet'
+        if self['ConsoleTypeDotted']:
+            return 'dotted'
+        return self['ConsoleType']
+
+    @property
+    def console_width(self):
+        return self['MonitorWidth'] or self['ConsoleWidth']
+
+    @property
+    def console_markers(self):
+        return self['MonitorMarkers'] or self['ConsoleMarkers']
+
+    @property
+    def pre_run_modifiers(self):
+        return self['PreRunModifiers']
+
+    @property
+    def run_empty_suite(self):
+        return self['RunEmptySuite']
+
+    @property
+    def variables(self):
+        return self['Variables']
+
+    @property
+    def variable_files(self):
+        return self['VariableFiles']
+
 
 class RebotSettings(_BaseSettings):
     _extra_cli_opts = {'Output'            : ('output', None),
@@ -480,8 +520,8 @@ class RebotSettings(_BaseSettings):
                        'ProcessEmptySuite' : ('processemptysuite', False),
                        'StartTime'         : ('starttime', None),
                        'EndTime'           : ('endtime', None),
-                       'ReRunMerge'        : ('rerunmerge', False),
-                       'Merge'             : ('merge', False)}
+                       'Merge'             : ('merge', False),
+                       'DeprecatedMerge'   : ('rerunmerge', False)}
 
     def _output_disabled(self):
         return False
@@ -497,7 +537,7 @@ class RebotSettings(_BaseSettings):
             'exclude_tags': self['Exclude'],
             'include_suites': self['SuiteNames'],
             'include_tests': self['TestNames'],
-            'empty_suite_ok': self['ProcessEmptySuite'],
+            'empty_suite_ok': self.process_empty_suite,
             'remove_keywords': self.remove_keywords,
             'log_level': self['LogLevel'],
             'critical_tags': self.critical_tags,
@@ -511,7 +551,7 @@ class RebotSettings(_BaseSettings):
         if not self.log:
             return {}
         return {
-            'title': utils.html_escape(self['LogTitle'] or ''),
+            'title': html_escape(self['LogTitle'] or ''),
             'reportURL': self._url_from_path(self.log, self.report),
             'splitLogBase': os.path.basename(os.path.splitext(self.log)[0]),
             'defaultLevel': self['VisibleLogLevel']
@@ -522,7 +562,7 @@ class RebotSettings(_BaseSettings):
         if not self.report:
             return {}
         return {
-            'title': utils.html_escape(self['ReportTitle'] or ''),
+            'title': html_escape(self['ReportTitle'] or ''),
             'logURL': self._url_from_path(self.report, self.log),
             'background' : self._resolve_background_colors(),
         }
@@ -530,7 +570,7 @@ class RebotSettings(_BaseSettings):
     def _url_from_path(self, source, destination):
         if not destination:
             return None
-        return utils.get_link_path(destination, os.path.dirname(source))
+        return get_link_path(destination, os.path.dirname(source))
 
     def _resolve_background_colors(self):
         colors = self['ReportBackground']
@@ -538,12 +578,16 @@ class RebotSettings(_BaseSettings):
 
     @property
     def merge(self):
-        return self['Merge'] or self['ReRunMerge']
+        return self['Merge'] or self['DeprecatedMerge']
 
     @property
-    def console_logger_config(self):
+    def console_output_config(self):
         return {
-            'colors':  self['MonitorColors'],
+            'colors':  self.console_colors,
             'stdout':  self['StdOut'],
             'stderr':  self['StdErr']
         }
+
+    @property
+    def process_empty_suite(self):
+        return self['ProcessEmptySuite']
