@@ -328,7 +328,9 @@ class _List(object):
         The keyword first verifies that the lists have equal lengths, and then
         it checks are all their values equal. Possible differences between the
         values are listed in the default error message like ``Index 4: ABC !=
-        Abc``.
+        Abc``. The types of the lists do not need to be the same. For example,
+        Python tuple and list with same content are considered equal.
+
 
         The error message can be configured using ``msg`` and ``values``
         arguments:
@@ -518,7 +520,7 @@ class _Dictionary(object):
         | ${keys} = ['a', 'b', 'c']
         """
         # TODO: Possibility to disable sorting. Can be handy with OrderedDicts.
-        return sorted(dictionary)
+        return sorted(dictionary.keys())
 
     def get_dictionary_values(self, dictionary):
         """Returns values of the given dictionary.
@@ -625,7 +627,8 @@ class _Dictionary(object):
 
         First the equality of dictionaries' keys is checked and after that all
         the key value pairs. If there are differences between the values, those
-        are listed in the error message.
+        are listed in the error message. The types of the dictionaries do not
+        need to be same.
 
         See `Lists Should Be Equal` for more information about configuring
         the error message with ``msg`` and ``values`` arguments.
