@@ -1,4 +1,4 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -62,8 +62,6 @@ class Screenshot(object):
     - Python Imaging Library (PIL) :: http://www.pythonware.com/products/pil ::
       This module can take screenshots only on Windows.
 
-    Python support was added in Robot Framework 2.5.5.
-
     = Using with Jython and IronPython =
 
     With Jython and IronPython this library uses APIs provided by JVM and .NET
@@ -79,32 +77,9 @@ class Screenshot(object):
     into the directory where the XML output file is written.
 
     It is possible to specify a custom location for screenshots using
-   `screenshot_directory` argument in `importing` and `Set Screenshot Directory`
-    keyword during execution. It is also possible to save screenshots using
-    an absolute path.
-
-    Note that prior to Robot Framework 2.5.5 the default screenshot location
-    was system's temporary directory.
-
-    = Changes in Robot Framework 2.5.5 and Robot Framework 2.6 =
-
-    This library was heavily enhanced in Robot Framework 2.5.5 release. The
-    changes are listed below and explained more thoroughly in affected places.
-
-    - The support for using this library on Python (see above) was added.
-    - The default location where screenshots are saved was changed (see above).
-    - New `Take Screenshot` and `Take Screenshot Without Embedding` keywords
-      were added. These keywords should be used for taking screenshots in
-      the future. Other screenshot taking keywords will be deprecated and
-      removed later.
-    - `log_file_directory` argument was deprecated everywhere it was used.
-
-    In Robot Framework 2.6, following additional changes were made:
-
-    - `log_file_directory` argument was removed altogether.
-    - `Set Screenshot Directories` keyword was removed.
-    - `Save Screenshot`, `Save Screenshot To` and `Log Screenshot`
-      keywords were deprecated. They will be removed in Robot Framework 2.8.
+   ``screenshot_directory`` argument when `importing` the library and
+    using `Set Screenshot Directory` keyword during execution. It is also
+    possible to save screenshots using an absolute path.
     """
 
     ROBOT_LIBRARY_SCOPE = 'TEST SUITE'
@@ -113,15 +88,15 @@ class Screenshot(object):
     def __init__(self, screenshot_directory=None):
         """Configure where screenshots are saved.
 
-        If `screenshot_directory` is not given, screenshots are saved into
+        If ``screenshot_directory`` is not given, screenshots are saved into
         same directory as the log file. The directory can also be set using
         `Set Screenshot Directory` keyword.
 
         Examples (use only one of these):
 
-        | =Setting= | =Value=  |  =Value=   | =Value= |
-        | Library | Screenshot |            | # Default location |
-        | Library | Screenshot | ${TEMPDIR} | # System temp (this was default prior to 2.5.5) |
+        | =Setting= |  =Value=   |  =Value=   |      =Value=       |
+        | Library   | Screenshot |            | # Default location |
+        | Library   | Screenshot | ${TEMPDIR} | # System temp      |
         """
         self._given_screenshot_dir = self._norm_path(screenshot_directory)
         self._screenshot_taker = ScreenshotTaker()
@@ -146,8 +121,8 @@ class Screenshot(object):
     def set_screenshot_directory(self, path):
         """Sets the directory where screenshots are saved.
 
-        It is possible to use `/` as a path separator in all operating systems.
-        Path to the old directory is returned.
+        It is possible to use ``/`` as a path separator in all operating
+        systems. Path to the old directory is returned.
 
         The directory can also be set in `importing`.
         """
@@ -162,16 +137,16 @@ class Screenshot(object):
         """Takes a screenshot in JPEG format and embeds it into the log file.
 
         Name of the file where the screenshot is stored is derived from the
-        given `name`. If the `name` ends with extension `.jpg` or `.jpeg`,
-        the screenshot will be stored with that exact name. Otherwise a unique
-        name is created by adding an underscore, a running index and
-        an extension to the `name`.
+        given ``name``. If the ``name`` ends with extension ``.jpg`` or
+        ``.jpeg``, the screenshot will be stored with that exact name.
+        Otherwise a unique name is created by adding an underscore, a running
+        index and an extension to the ``name``.
 
         The name will be interpreted to be relative to the directory where
         the log file is written. It is also possible to use absolute paths.
-        Using `/` as a path separator works in all operating systems.
+        Using ``/`` as a path separator works in all operating systems.
 
-        `width` specifies the size of the screenshot in the log file.
+        ``width`` specifies the size of the screenshot in the log file.
 
         Examples: (LOGDIR is determined automatically by the library)
         | Take Screenshot |                  |     | # LOGDIR/screenshot_1.jpg (index automatically incremented) |
@@ -310,7 +285,7 @@ class ScreenshotTaker(object):
 
     def _wx_screenshot(self, path):
         if not self._wx_app_reference:
-            self._wx_app_reference = wx.PySimpleApp()
+            self._wx_app_reference = wx.App(False)
         context = wx.ScreenDC()
         width, height = context.GetSize()
         bitmap = wx.EmptyBitmap(width, height, -1)
