@@ -1,4 +1,4 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -34,9 +34,7 @@ Examples
 --------
 
 First, let's assume we have the following test suite in file
-``activate_skynet.txt``:
-
-.. sourcecode:: RobotFramework
+``activate_skynet.robot``::
 
     *** Settings ***
     Library    OperatingSystem
@@ -52,7 +50,7 @@ using the :class:`~robot.running.builder.TestSuiteBuilder` class as follows::
 
     from robot.api import TestSuiteBuilder
 
-    suite = TestSuiteBuilder().build('path/to/activate_skynet.txt')
+    suite = TestSuiteBuilder().build('path/to/activate_skynet.robot')
 
 That was easy. Let's next generate the same test suite from scratch
 using the :class:`~robot.running.model.TestSuite` class::
@@ -91,7 +89,7 @@ the results is possible using the
 
     from robot.api import ResultWriter
 
-    # Report and xUnit files can be generated based on the  result object.
+    # Report and xUnit files can be generated based on the result object.
     ResultWriter(result).write_results(report='skynet.html', log=None)
     # Generating log files requires processing the earlier generated output XML.
     ResultWriter('skynet.xml').write_results()
@@ -100,10 +98,9 @@ Package methods
 ---------------
 """
 
-from .builder import TestSuiteBuilder
+from .builder import TestSuiteBuilder, ResourceFileBuilder
 from .context import EXECUTION_CONTEXTS
-from .keywords import Keyword
-from .model import TestSuite, TestCase
+from .model import Keyword, TestCase, TestSuite
 from .testlibraries import TestLibrary
 from .runkwregister import RUN_KW_REGISTER
 
@@ -113,19 +110,17 @@ def UserLibrary(path):
 
     This is used by Libdoc.
     """
-    from robot.parsing import ResourceFile
     from robot import utils
-    from .arguments.argumentspec import ArgumentSpec
+    from .arguments import ArgumentSpec
     from .userkeyword import UserLibrary as RuntimeUserLibrary
 
-    resource = ResourceFile(path).populate()
-    ret = RuntimeUserLibrary(resource.keyword_table.keywords, path)
-    for handler in ret.handlers.values():
+    resource = ResourceFileBuilder().build(path)
+    ret = RuntimeUserLibrary(resource.keywords, path)
+    for handler in ret.handlers:
         if handler.type != 'error':
             handler.doc = utils.unescape(handler._doc)
         else:
             handler.arguments = ArgumentSpec(handler.longname)
             handler.doc = '*Creating keyword failed: %s*' % handler.error
-    ret.doc = resource.setting_table.doc.value
+    ret.doc = utils.unescape(resource.doc)
     return ret
-
