@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -106,6 +106,9 @@ Alternative execution
 In the above examples Tidy is used only with Python, but it works also with
 Jython and IronPython. Above it is executed as an installed module, but it
 can also be run as a script like `python path/robot/tidy.py`.
+
+For more information about Tidy and other built-in tools, see
+http://robotframework.org/robotframework/#built-in-tools.
 """
 
 import os
@@ -233,7 +236,8 @@ class TidyCommandLine(Application):
 
     def validate(self, opts, args):
         validator = ArgumentValidator()
-        validator.mode_and_arguments(args, **opts)
+        opts['recursive'], opts['inplace'] \
+            = validator.mode_and_arguments(args, **opts)
         opts['format'] = validator.format(args, **opts)
         opts['lineseparator'] = validator.line_sep(**opts)
         if not opts['spacecount']:
@@ -246,11 +250,14 @@ class TidyCommandLine(Application):
 class ArgumentValidator(object):
 
     def mode_and_arguments(self, args, recursive, inplace, **others):
+        recursive, inplace = bool(recursive), bool(inplace)
         validators = {(True, True): self._recursive_and_inplace_together,
                       (True, False): self._recursive_mode_arguments,
                       (False, True): self._inplace_mode_arguments,
                       (False, False): self._default_mode_arguments}
-        validators[(recursive, inplace)](args)
+        validator = validators[(recursive, inplace)]
+        validator(args)
+        return recursive, inplace
 
     def _recursive_and_inplace_together(self, args):
         raise DataError('--recursive and --inplace can not be used together.')
