@@ -1,4 +1,4 @@
-#  Copyright 2008-2014 Nokia Solutions and Networks
+#  Copyright 2008-2015 Nokia Solutions and Networks
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -12,11 +12,16 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-import time
 import datetime
+import time
+import re
 
 from .normalizing import normalize
 from .misc import plural_or_not
+from .robottypes import is_number, is_string
+
+
+_timer_re = re.compile('([+-])?(\d+:)?(\d+):(\d+)(.\d+)?')
 
 
 def _get_timetuple(epoch_secs=None):
@@ -32,38 +37,39 @@ def _float_secs_to_secs_and_millis(secs):
     return (isecs, millis) if millis < 1000 else (isecs+1, 0)
 
 
-START_TIME = _get_timetuple()
+def timestr_to_secs(timestr, round_to=3):
+    """Parses time like '1h 10s', '01:00:10' or '42' and returns seconds."""
+    if is_string(timestr) or is_number(timestr):
+        for converter in _number_to_secs, _timer_to_secs, _time_string_to_secs:
+            secs = converter(timestr)
+            if secs is not None:
+                return secs if round_to is None else round(secs, round_to)
+    raise ValueError("Invalid time string '%s'." % timestr)
 
-
-def timestr_to_secs(timestr):
-    """Parses time in format like '1h 10s' and returns time in seconds (float).
-
-    Given time must be in format '1d 2h 3m 4s 5ms' with following rules:
-
-    - Time parts having zero value can be ignored (e.g. '3m 4s' is ok)
-    - Format is case and space insensitive
-    - Instead of 'd' it is also possible to use 'day' or 'days'
-    - Instead of 'h' also 'hour' and 'hours' are ok
-    - Instead of 'm' also 'minute', 'minutes', 'min' and 'mins' are ok
-    - Instead of 's' also 'second', 'seconds', 'sec' and 'secs' are ok
-    - Instead of 'ms' also 'millisecond', 'milliseconds' and 'millis' are ok
-    - It is possible to give time only as a float and then it is considered
-      to be seconds (e.g. '123', '123.0', '123s', '2min 3s' are all equivelant)
-    """
+def _number_to_secs(number):
     try:
-        secs = _timestr_to_secs(timestr)
-    except (ValueError, TypeError):
-        raise ValueError("Invalid time string '%s'" % timestr)
-    return round(secs, 3)
-
-def _timestr_to_secs(timestr):
-    timestr = _normalize_timestr(timestr)
-    if timestr == '':
-        raise ValueError
-    try:
-        return float(timestr)
+        return float(number)
     except ValueError:
-        pass
+        return None
+
+def _timer_to_secs(number):
+    match = _timer_re.match(number)
+    if not match:
+        return None
+    prefix, hours, minutes, seconds, millis = match.groups()
+    seconds = float(minutes) * 60 + float(seconds)
+    if hours:
+        seconds += float(hours[:-1]) * 60 * 60
+    if millis:
+        seconds += float(millis[1:]) / 10**len(millis[1:])
+    if prefix == '-':
+        seconds *= -1
+    return seconds
+
+def _time_string_to_secs(timestr):
+    timestr = _normalize_timestr(timestr)
+    if not timestr:
+        return None
     millis = secs = mins = hours = days = 0
     if timestr[0] == '-':
         sign = -1
@@ -72,33 +78,32 @@ def _timestr_to_secs(timestr):
         sign = 1
     temp = []
     for c in timestr:
-        if   c == 'x': millis = float(''.join(temp)); temp = []
-        elif c == 's': secs   = float(''.join(temp)); temp = []
-        elif c == 'm': mins   = float(''.join(temp)); temp = []
-        elif c == 'h': hours  = float(''.join(temp)); temp = []
-        elif c == 'p': days   = float(''.join(temp)); temp = []
-        else: temp.append(c)
+        try:
+            if   c == 'x': millis = float(''.join(temp)); temp = []
+            elif c == 's': secs   = float(''.join(temp)); temp = []
+            elif c == 'm': mins   = float(''.join(temp)); temp = []
+            elif c == 'h': hours  = float(''.join(temp)); temp = []
+            elif c == 'd': days   = float(''.join(temp)); temp = []
+            else: temp.append(c)
+        except ValueError:
+            return None
     if temp:
-        raise ValueError
+        return None
     return sign * (millis/1000 + secs + mins*60 + hours*60*60 + days*60*60*24)
 
 def _normalize_timestr(timestr):
-    if isinstance(timestr, (int, long, float)):
-        return timestr
     timestr = normalize(timestr)
-    for item in 'milliseconds', 'millisecond', 'millis':
-        timestr = timestr.replace(item, 'ms')
-    for item in 'seconds', 'second', 'secs', 'sec':
-        timestr = timestr.replace(item, 's')
-    for item in 'minutes', 'minute', 'mins', 'min':
-        timestr = timestr.replace(item, 'm')
-    for item in 'hours', 'hour':
-        timestr = timestr.replace(item, 'h')
-    for item in 'days', 'day':
-        timestr = timestr.replace(item, 'd')
-    # 1) 'ms' -> 'x' to ease processing later
-    # 2) 'd' -> 'p' because float('1d') returns 1.0 in Jython (bug submitted)
-    return timestr.replace('ms','x').replace('d','p')
+    for specifier, aliases in [('x', ['millisecond', 'millisec', 'millis',
+                                      'msec', 'ms']),
+                               ('s', ['second', 'sec']),
+                               ('m', ['minute', 'min']),
+                               ('h', ['hour']),
+                               ('d', ['day'])]:
+        plural_aliases = [a+'s' for a in aliases if not a.endswith('s')]
+        for alias in plural_aliases + aliases:
+            if alias in timestr:
+                timestr = timestr.replace(alias, specifier)
+    return timestr
 
 
 def secs_to_timestr(secs, compact=False):
@@ -171,7 +176,7 @@ def format_time(timetuple_or_epochsecs, daysep='', daytimesep=' ', timesep=':',
 
     Seconds after epoch can be either an integer or a float.
     """
-    if isinstance(timetuple_or_epochsecs, (int, long, float)):
+    if is_number(timetuple_or_epochsecs):
         timetuple = _get_timetuple(timetuple_or_epochsecs)
     else:
         timetuple = timetuple_or_epochsecs
@@ -319,10 +324,6 @@ def secs_to_timestamp(secs, seps=None, millis=False):
     return format_time(ttuple, *seps)
 
 
-def get_start_timestamp(daysep='', daytimesep=' ', timesep=':', millissep=None):
-    return format_time(START_TIME, daysep, daytimesep, timesep, millissep)
-
-
 def get_elapsed_time(start_time, end_time):
     """Returns the time between given timestamps in milliseconds."""
     if start_time == end_time or not (start_time and end_time):
